@@ -23,7 +23,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #endif
 #define _Integer_h 1
 
-#include <stream.h>
+#include <iostream.h>
 
 struct IntRep                    // internal Integer representations
 {
@@ -39,6 +39,7 @@ struct IntRep                    // internal Integer representations
 
 extern IntRep*  Ialloc(IntRep*, const unsigned short *, int, int, int);
 extern IntRep*  Icalloc(IntRep*, int);
+extern IntRep*  Icopy_ulong(IntRep*, unsigned long);
 extern IntRep*  Icopy_long(IntRep*, long);
 extern IntRep*  Icopy(IntRep*, const IntRep*);
 extern IntRep*  Iresize(IntRep*, int);
@@ -83,7 +84,9 @@ protected:
   IntRep*         rep;
 public:
                   Integer();
+                  Integer(int);
                   Integer(long);
+                  Integer(unsigned long);
                   Integer(IntRep*);
                   Integer(const Integer&);
 
@@ -142,7 +145,7 @@ public:
   friend int      odd(const Integer&); // true if odd
   friend int      sign(const Integer&); // returns -1, 0, +1
 
-  friend void     setbit(Integer& x, long b);   // set b'th bit of x
+  friend void     (setbit)(Integer& x, long b);   // set b'th bit of x
   friend void     clearbit(Integer& x, long b); // clear b'th bit
   friend int      testbit(const Integer& x, long b);  // return b'th bit
 
@@ -194,11 +197,16 @@ public:
 
 // coercion & conversion
 
-  int             fits_in_long() const;
-  int             fits_in_double() const;
+  int             fits_in_long() const { return Iislong(rep); }
+  int             fits_in_double() const { return Iisdouble(rep); }
 
-                  operator long() const;
-                  operator double() const;
+#if 0
+  // There two operators cause a number of ambiguities.
+                  operator long() const { return Itolong(rep); }
+                  operator double() const { return Itodouble(rep); }
+#endif
+  long		  as_long() const { return Itolong(rep); }
+  double	  as_double() const { return Itodouble(rep); }
 
   friend char*    Itoa(const Integer& x, int base = 10, int width = 0);
   friend Integer  atoI(const char* s, int base = 10);
@@ -279,8 +287,12 @@ inline Integer::Integer() :rep(&_ZeroRep) {}
 
 inline Integer::Integer(IntRep* r) :rep(r) {}
 
+inline Integer::Integer(int y) :rep(Icopy_long(0, (long)y)) {}
+
 inline Integer::Integer(long y) :rep(Icopy_long(0, y)) {}
 
+inline Integer::Integer(unsigned long y) :rep(Icopy_ulong(0, y)) {}
+
 inline Integer::Integer(const Integer&  y) :rep(Icopy(0, y.rep)) {}
 
 inline Integer::~Integer() { if (rep && !STATIC_IntRep(rep)) delete rep; }
@@ -295,31 +307,11 @@ inline void Integer::operator = (long y)
   rep = Icopy_long(rep, y); 
 }
 
-inline Integer::operator long() const
-{ 
-  return Itolong(rep);
-}
-
 inline int Integer::initialized() const
 {
   return rep != 0;
 }
 
-inline int Integer::fits_in_long() const
-{
-  return Iislong(rep);
-}
-
-inline Integer::operator double() const
-{ 
-  return Itodouble(rep);
-}
-
-inline int Integer::fits_in_double() const
-{
-  return Iisdouble(rep);
-}
-
 // procedural versions
 
 inline int compare(const Integer& x, const Integer& y)
@@ -404,7 +396,7 @@ inline void  rshift(const Integer& x, const Integer& y, Integer& dest)
 
 inline void  pow(const Integer& x, const Integer& y, Integer& dest)
 {
-  dest.rep = power(x.rep, long(y), dest.rep); // not incorrect
+  dest.rep = power(x.rep, Itolong(y.rep), dest.rep); // not incorrect
 }
 
 inline void  add(const Integer& x, long y, Integer& dest)
@@ -659,17 +651,6 @@ inline void Integer::operator /= (long y)
 }
 
 
-inline void Integer::operator %= (const Integer& y)
-{
-  mod(*this, y, *this);
-}
-
-inline void Integer::operator %= (long y)
-{
-  mod(*this, y, *this);
-}
-
-
 inline void Integer::operator <<= (const Integer&  y)
 {
   lshift(*this, y, *this);
@@ -1109,4 +1090,14 @@ inline Integer  gcd(const Integer& x, const Integer& y)
 }
 
 #endif  /* NO_NRV */
+
+inline void Integer::operator %= (const Integer& y)
+{
+  *this = *this % y; // mod(*this, y, *this) doesn't work.
+}
+
+inline void Integer::operator %= (long y)
+{
+  *this = *this % y; // mod(*this, y, *this) doesn't work.
+}
 #endif /* !_Integer_h */
